'*************************************************************
'    by Wolfgang Hugemann, autorenteam GbR, Muenster, Germany
'   www.unfallrekonstruktion.eu
'
' This script maps one image onto another by means of ImageMagick's perspective distortion www.imagemagick.org).
' It uses a WinMorph (www.debugmode.com/winmorph) data file as input.
' Amongst other things, the script can perform a perspective rectification, 
' if the target image is planar view on the taget object.
'
' The only command line argument of the script in the name of the WinMorph data file (*.mrf),
' from which the coordinates of the source and taget points are read, as well as the image filenames. 
'  The projected (rectified) image will have the same filename as the original, with '_pr' (perspectively rectified)
'  attached to it. The file extension will be the same as that of the input file.
'
' The source and target images must be in the same directory as the WinMorph file. 
' IF YOUR COUNTRY USES A DECIMAL COMMA INSTEAD OF A DECIMAL POINT,
' you have to uncomment the lines that contain the "ReplaceStr" statements.
'
' You need at least four points to perform the perspective transformation. If
' you supply exactly four points, the images of these points in the perspectivly transformed image
' will exactly come to lie on the target points. (You can check this by overlaying the tranformed image and the
' target image.) ImageMagick can however perform a perspective transfomation based on more than four points.
' In this case, a least squares approach is applied and none of the transformed points will exactly come to
' lie on its target point.
' 
'  Version 2008-07-02
''*************************************************************
Option Explicit
' ImageMagick commands
Const strConv = "IMconvert"   ' Filename of the Convert command line tool
const strPR   = "-virtual-pixel white -distort perspective """ ' command line stub
const strAdd = "_pr"          ' added to the   output fimename
const strExt  = ".miff"       ' filename extension for interim files

Dim strTempFile1, strTempFile2
strTempFile1 = "canvas.miff"   ' temporary filename #1
strTempFile2 = "extended.miff" ' temporary filename #2

Dim wsh, Fso, objFile, objExec
Dim IOFile(2)

Dim i, j, k, Pos, nPoints 'Integer
Dim isWmFile, isRead ' Boolean
Dim strWmPath, strWmFileName, strLine, strFile, strCommand 'strings

Set wsh = CreateObject("Wscript.Shell")
Set fso = CreateObject("Scripting.FileSystemObject")
'
' check whether a command line parameter (CLP) was handed to this script
If WScript.Arguments.count = 0 then
	MsgBox "No filename given.", vbExclamation, "Abort"
	Wscript.Quit(1)
end if
'
' check whether the CLP is a valid filename
strWmFileName = WScript.Arguments(0)
If not fso.FileExists(strWmFileName) then
	MsgBox """" & strWmFileName & """ is no valid file.", vbExclamation, "Abort"
	Wscript.Quit(1)
end if
'
' check whether the CLP is a WinMorh file
if Ucase(fso.GetExtensionName(strWmFilename)) <> "MRF" then
	MsgBox """" & strWmFileName & """ is not a WinMorph file.", _
	   vbExclamation, "Abort"
	Wscript.Quit(1)
end if
'
' The WinMorph file lists only the bare filenames (without the path) 
' => store the path of the WinMorph file in order to address the images correctly
Pos = InStrRev(strWmFileName,"\")
strWmpath = Left(strWmFileName, Pos)
'
' The element "Filename" exists only two times in the WinMorph file:
' Its first appearance names the input image filename, the second one the output image filename
Set objFile = fso.OpenTextFile(strWmFileName, 1)

IOFile(0) = ""
IOFile(1) = ""
Do While IOFile(1) = ""
	strLine = objFile.ReadLine
	Pos = InStr(strLine,"Filename")
	if Pos > 0 then
		Pos=Instr(strLine,"=")
		strFile = Trim(Mid(strLine, Pos + 1))
		If IOFile(0) = "" then
			IOFile(0) = """" & strWmpath & strFile & """"
		else
			IOFile(1) = """" & strWmpath & strFile & """"
		end if
	end if
Loop
'
' Read the coordinates of the point pairs (i.e. source point and according target point)
' The elements containig this information are called "Shape1" and "Shape2", 
' followed by two lines with control data and then followed by one point per line
' which is given in relative coordinates (real values)
' x = b/B -- y = h/H
'
' first field index: point number (maximum of 10)
' fsecond field index:   1 = Input, 2 = Ouptut
' third field index: 1= x, 2 = y
'
Dim Points(10,2,2)
Dim strObj(2)
strObj(0) = "Shape1"
strObj(1) = "Shape2"

for j = 0 to 1
	i = 0
	isRead = false
	Do While not isRead
		strLine = objFile.ReadLine
		Pos = InStr(strLine,strObj(j))
		if Pos > 0 then
			isRead = True
			objFile.ReadLine
			objFile.ReadLine
			Do
				strLine = Replace(objFile.Readline, vbTab,"")
				If not isNumeric(left(strLine,1)) then exit do
				pos = Instr(strLine, " ")
				Points(i,j,0) = Left(strLine, Pos - 1)
				Points(i,j,1) =  Mid(strline, Pos + 1)
				i = i + 1
			Loop
		end if
	Loop
next
objFile.close
nPoints = i
'
' Detemine width and height of source and target image
' first field index:   1 = Input, 2 = Ouptut
' second field index: 1= x, 2 = y
'
Dim PicDim (2,2)
for i = 0 to 1
	strCommand = "identify -format %wx%h " & IOFile(i)
	Set objExec = wsh.Exec(strCommand)
	strLine = objExec.StdOut.Readline()
	Pos = Instr(strLine,"x")
	PicDim(i,0) = Left(strLine,Pos - 1)
	PicDim(i,1) =  Mid(strLine,Pos + 1)
Next
'
' extend the source image to the dimensions of the target image
' such that the perspectively transformed image will later exactly have the
' dimensions of the target image (and can be overlayed)
strTempFile1 = """" & strWmPath & strTempFile1 & """"
strTempFile2 = """" & strWmPath & strTempFile2 & """"
strCommand = strConv & " -size " & PicDim(1,0) & "x" & PicDim(1,1) & " xc:white " & strTempfile1
wsh.run strcommand, 7, true
strcommand = "composite -gravity northwest " & IoFile(0) & " " & strTempfile1 & " " & strTempfile2
wsh.run strcommand, 7, true

'
' Transform relative coordinates in pixel coordinates
' the conversion of strings into numbers seems to be region-specific
' On a German computer, the decimal comma has to be substituted by a decimal point
For i = 0 to npoints - 1
	For j = 0 to 1
		For k = 0 to 1
			'Points(i,j,k) = int(Replace(Points(i,j,k), ".", ",") * PicDim(j,k) + 0.5)
			Points(i,j,k) = int(Points(i,j,k) * PicDim(j,k) + 0.5)
		next
	next
next
'
'strFile = output filename = input filename extended by "_pr" (strPR)
Pos = InStrRev(IOFile(0),".")
strFile = Left(IOFile(0),Pos - 1) & strAdd & Mid(IOfile(0), Pos)
strCommand  = strConv  & " " & strTempFile2 & " " & strPR
For i = 0 to npoints - 1
	For j = 0 to 1
		strCommand = strCommand & Points(i,j,0) & "," & Points(i,j,1) & " "
	next
next
strCommand = strCommand & """ " & strFile
wsh.run strcommand, 7, true
fso.DeleteFile(strWmPath & "*" & strExt)